-- ALU 
-- fichero: ALU.vhd
library IEEE; 
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.ALU_CONSTANTS.all;

entity ALU is 
   port (  y    : out std_logic_vector (WIDTH-1 downto 0);
           a, b : in  std_logic_vector (WIDTH-1 downto 0);
           cin  : in  std_logic_vector (0 downto 0); 
           sel  : in  std_logic_vector (SEL_BITS-1 downto 0) );
end entity ALU;

architecture ALU_concurrente of ALU is

   signal arith, logic: std_logic_vector(WIDTH-1 downto 0);

begin
   --Unidad Aritmtica
   with sel( SEL_BITS-2 downto 0 ) select
      arith <= a                                          when "000",
               std_logic_vector( signed(a) + 1 )          when "001",
               std_logic_vector( signed(a) - 1 )          when "010",
               b                                          when "011",
               std_logic_vector( signed(b) + 1 )          when "100",
               std_logic_vector( signed(b) - 1 )          when "101",  
               std_logic_vector( signed(a) + signed(b) )  when "110",
               std_logic_vector( signed(a) + signed(b) +
                                 signed("000" & cin)    ) when others;
                   
   --Unidad Lgica
   with sel( SEL_BITS-2 downto 0 ) select
      logic <= not a     when "000",
               not b     when "001",
               a and b   when "010",
               a or b    when "011",
               a nand b  when "100",
               a nor b   when "101",  
               a xor b   when "110",
               a xnor b  when others;   
               
   --Multiplexor
   with sel(SEL_BITS-1) select
      y <= arith when '0',
           logic when others;
                                    
end architecture ALU_concurrente;