-- Banco de pruebas para la ALU
-- fichero: bp_ALU.vhd
library IEEE; 
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.bp_ALU_CONSTANTS.all;
use work.ALU_CONSTANTS.all;

entity bp_ALU is 
end entity bp_ALU;

architecture bp_ALU of bp_ALU is
   signal a,b,y : std_logic_vector (WIDTH-1 downto 0);
   signal cin   : std_logic_vector (0 downto 0);
   signal sel   : std_logic_vector (SEL_BITS-1 downto 0);
  
   component ALU is 
      port (  y    : out std_logic_vector(WIDTH-1 downto 0);
              a, b : in std_logic_vector (WIDTH-1 downto 0);
              cin  : in std_logic_vector (0 downto 0); 
              sel  : in std_logic_vector (SEL_BITS-1 downto 0) );
   end component ALU;
  

   -- Procedure que calcula C (expected_C) y lo compara con el
   -- valor de C que se pasa como argumento (actual_C)
   -- Si ambos valores no coinciden, se muestra un mensaje y se
   -- incrementa el contador de errores (error_count)
   procedure check_ALU (
             a, b, cin, sel : in integer;
             actual_y       : in std_logic_vector (WIDTH-1 downto 0);
             error_count    : inout integer ) is 
      variable expected_y : integer;
   begin
      case sel is
         when 0      => expected_y := a;
         when 1      => expected_y := a+1;
         when 2      => expected_y := a-1;
         when 3      => expected_y := b;
         when 4      => expected_y := b+1;
         when 5      => expected_y := b-1;
         when 6      => expected_y := a+b;
         when 7      => expected_y := a+b+cin;
         when 8      => expected_y := TO_INTEGER(signed( 
                          not std_logic_vector(TO_SIGNED(a,WIDTH)) ));
         when 9      => expected_y := TO_INTEGER(signed( 
                          not std_logic_vector(TO_SIGNED(b,WIDTH)) ));
         when 10     => expected_y := TO_INTEGER(signed(
                          std_logic_vector(TO_SIGNED(a,WIDTH)) and  
                          std_logic_vector(TO_SIGNED(b,WIDTH))     ));
         when 11     => expected_y := TO_INTEGER(signed(
                          std_logic_vector(TO_SIGNED(a,WIDTH)) or  
                          std_logic_vector(TO_SIGNED(b,WIDTH))     ));
         when 12     => expected_y := TO_INTEGER(signed(
                          std_logic_vector(TO_SIGNED(a,WIDTH)) nand  
                          std_logic_vector(TO_SIGNED(b,WIDTH))     ));
         when 13     => expected_y := TO_INTEGER(signed(
                          std_logic_vector(TO_SIGNED(a,WIDTH)) nor  
                          std_logic_vector(TO_SIGNED(b,WIDTH))     ));  
         when 14     => expected_y := TO_INTEGER(signed(
                          std_logic_vector(TO_SIGNED(a,WIDTH)) xor  
                          std_logic_vector(TO_SIGNED(b,WIDTH))     ));                             
         when others => expected_y := TO_INTEGER(signed(
                          std_logic_vector(TO_SIGNED(a,WIDTH)) xnor  
                          std_logic_vector(TO_SIGNED(b,WIDTH))     ));     
      end case;
      expected_y := TO_INTEGER(TO_SIGNED(expected_y,WIDTH));
                    -- Trunca el resultado a WIDTH bits
      assert( expected_y = TO_INTEGER(SIGNED(actual_y))  )
         report "ERROR. Operandos: "                         & 
                integer'image(a)                             & 
                ", " & integer'image(b)                      &
                ", " & integer'image(cin)                    &
                ", Operacion: "                              & 
                integer'image(sel)                           &
                ", resultado esperado: "                     &
                integer'image(expected_y)                    &
                ", resultado actual: "                       &
                integer'image(TO_INTEGER(signed(actual_y)))  &
                " en el instante "                           &
                time'image(now);
      if (expected_y /= TO_INTEGER(signed(actual_y))) then
         error_count := error_count + 1;
      end if;
   end procedure check_ALU;
   -- Fin de la definicin del procedure

  
begin

   UUT : component ALU port map (y, a, b, cin, sel);

   -- bloque process para generar los vectores de test y
   -- comprobar el resultado
   main : process is
      variable error_count : integer := 0;
   begin  
      report "Comienza la simulacin";
    
      -- Vectores de test: operandos con valor prximo a cero
      for i in -2 to 2 loop
         for j in -2 to 2 loop
            for k in 0 to SEL_MAX loop
               for l in 0 to 1 loop
                  a    <= std_logic_vector(TO_SIGNED(i,WIDTH));
                  b    <= std_logic_vector(TO_SIGNED(j,WIDTH));
                  sel  <= std_logic_vector(TO_SIGNED(k,SEL_BITS));
                  cin  <= std_logic_vector(TO_SIGNED(l,1));
                  wait for DELAY;
                  check_ALU(i, j, l, k, y, error_count);
               end loop;
            end loop;
         end loop;
      end loop;
    
      -- Vectores de test: operandos con valor prximo al mnimo
      for i in MIN_NEG_DATA to MIN_NEG_DATA+4 loop
         for j in MIN_NEG_DATA to MIN_NEG_DATA+4 loop
            for k in 0 to SEL_MAX loop
               for l in 0 to 1 loop
                  A    <= std_logic_vector(TO_SIGNED(i,WIDTH));
                  b    <= std_logic_vector(TO_SIGNED(j,WIDTH));
                  sel  <= std_logic_vector(TO_SIGNED(k,SEL_BITS));
                  cin  <= std_logic_vector(TO_SIGNED(l,1));          
                  wait for DELAY;
                  check_ALU(i, j, l, k, y, error_count);
               end loop;
            end loop;
         end loop;
      end loop;
    
      -- Vectores de test: operandos con valor prximo al mximo
      for i in MAX_POS_DATA-4 to MAX_POS_DATA loop
         for j in MAX_POS_DATA-4 to MAX_POS_DATA loop
            for k in 0 to SEL_MAX loop
               for l in 0 to 1 loop
                  a    <= std_logic_vector(TO_SIGNED(i,WIDTH));
                  b    <= std_logic_vector(TO_SIGNED(j,WIDTH));
                  sel  <= std_logic_vector(TO_SIGNED(k,SEL_BITS));
                  cin  <= std_logic_vector(TO_SIGNED(l,1));          
                  wait for DELAY;
                  check_ALU(i, j, l, k, y, error_count);
               end loop;
            end loop;
         end loop;
      end loop;


      -- Vectores de test: operandos con valores "al azar"
      for i in 0 to 9 loop
         for j in 0 to 9 loop
            for k in 0 to SEL_MAX loop
               for l in 0 to 1 loop
                  A    <= std_logic_vector(TO_SIGNED(41*i-273,WIDTH));
                  b    <= std_logic_vector(TO_SIGNED(89*j-384,WIDTH));
                  sel  <= std_logic_vector(TO_SIGNED(k,SEL_BITS));
                  cin  <= std_logic_vector(TO_SIGNED(l,1));          
                  wait for DELAY;
                  check_ALU(41*i-273, 89*j-384, l, k, y, error_count);
               end loop;
            end loop;
         end loop;
      end loop;

      wait for DELAY;
    
      -- Informe mostrando el resultado del test
      if (error_count=0) then
         report "Finaliza la simulacin: 0 errores";
      else
         report "Finaliza la simulacin: "   & 
                integer'image(error_count)   &
                " errores";
      end if;   
        
      wait;  -- Termina la simulacin
   end process main;
   
end architecture bp_ALU;
