-- Banco de pruebas del sumador paralelo de 4 bits
-- fichero: bp_sum_Nbits.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity bp_sum_Nbits is     -- El banco de pruebas no tiene
end entity bp_sum_Nbits;   -- ni entradas ni salidas

architecture bp_sum_Nbits of bp_sum_Nbits is
   -- Seales para conectar el UUT
   signal   C_out,C_in : std_logic_vector (0 downto 0);
   signal   x, y, s    : std_logic_vector (3 downto 0);  
  
   component sumador_Nbits is 
      generic ( N: integer :=4 );
      port ( s     : out std_logic_vector (N-1 downto 0);
             C_out : out std_logic;
             x, y  : in  std_logic_vector (N-1 downto 0);
             C_in  : in  std_logic);
   end component sumador_Nbits;

begin

   -- Instanciar y conectar UUT
   uut : component sumador_Nbits port map
         (  s => s, C_out => C_out(0),
            x => x, y => y, C_in => C_in(0) );

   -- Crear vectores de test y comprobar salidas del UUT
   gen_vec_test : process
      variable sumaEsperada : unsigned (4 downto 0); -- Resultado suma
      variable num_errores  : integer := 0;          -- Nm. errores
   begin
      for sumando_X in 0 to 2**4-1 loop
         for sumando_Y in 0 to 2**4-1 loop
            for acarreo_in in 0 to 1 loop
               sumaEsperada := to_unsigned(acarreo_in,5) + 
                               to_unsigned(sumando_x,5)  + 
                               to_unsigned(sumando_y,5);
               x    <= std_logic_vector(to_unsigned(sumando_x,4));
               y    <= std_logic_vector(to_unsigned(sumando_y,4));
               C_in <= std_logic_vector(to_unsigned(acarreo_in,1));
               wait for 100 ns;
               report time'image(now)  &  ": "     &
                  std_logic'image(sumaEsperada(4)) & 
                  std_logic'image(sumaEsperada(3)) &
                  std_logic'image(sumaEsperada(2)) &
                  std_logic'image(sumaEsperada(1)) &
                  std_logic'image(sumaEsperada(0)) &
                  " Suma calculada : "             &
                  std_logic'image(C_out(0))        &
                  std_logic'image(s(3))            &
                  std_logic'image(s(2))            &
                  std_logic'image(s(1))            &
                  std_logic'image(s(0));
               -- Comprueba resultado
               if ( sumaEsperada /= unsigned( C_out & s ) ) then     
                  num_errores := num_errores + 1;
               end if;
            end loop;
         end loop;
      end loop;

      report "Test completo. Hay "      &
             integer'image(num_errores) & " errores.";
      wait;
   end process gen_vec_test;
end architecture bp_sum_Nbits;
