-- Banco de pruebas del desplazador
-- fichero: bp_desplazador.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity bp_desplazador is
   constant DELAY : time := 10 ns; -- Retardo usado en el test
end entity bp_desplazador;

architecture bp_desplazador of bp_desplazador is

   signal salida  : std_logic_vector(3 downto 0); -- Salida UUT
   signal op      : std_logic_vector(1 downto 0); -- Entradas UUT
   signal entrada : std_logic_vector(3 downto 0);
  
   component desplazador is
      port ( salida  : out std_logic_vector (3 downto 0);
             op      : in  std_logic_vector (1 downto 0);
             entrada : in  std_logic_vector (3 downto 0) );       
   end component desplazador;

   -- Procedure que calcula la salida (expected_s) y lo compara con el
   -- valor de salida que se pasa como argumento (actual_s)
   -- Si ambos valores no coinciden, se muestra un mensaje y se
   -- incrementa el contador de errores (error_count)
   procedure check_desp (
                op         : in integer;
                data       : in unsigned (3 downto 0);
                actual_s   : in unsigned (3 downto 0);
                error_count: inout integer ) is 
      variable expected_s : unsigned(3 downto 0);
   begin
      case op is
         when 0      => expected_s := shift_left(data,1);
         when 1      => expected_s := shift_right(data,1);
         when 2      => expected_s := rotate_left(data,1);
         when others => expected_s := rotate_right(data,1);
      end case;

      -- Las funciones shift_left, shift_right, rotate_left y 
      -- rotate_right se encuentran en el paquete IEEE.numeric_std  
      assert( expected_s = actual_s)
         report "ERROR. Entrada: "                    & 
                integer'image(to_integer(data))       & 
                ", Operacion: " & integer'image(op)   &
                ", resultado esperado: "              &
                integer'image(to_integer(expected_s)) &
                ", resultado actual: "                &
                integer'image(to_integer(actual_s))   &
                " en el instante "                    &
                time'image(now);
              
      if ( expected_s /= actual_s ) then
         error_count := error_count + 1;
      end if;
    
   end procedure check_desp;  -- Fin definicin del procedure

begin  

   UUT : component desplazador port map
         (salida, op, entrada);
    
   vec_test : process is
      variable temp        : std_logic_vector (5 downto 0);
      variable error_count : integer := 0;
   begin
      report "Comienza la simulacin";
      -- Generar todos los posibles valores de entrada
      for i in 0 to 2**6-1 loop
         temp    := std_logic_vector( to_unsigned(i,6) );
         op      <= temp(5 downto 4);
         entrada <= temp(3 downto 0);
         wait for DELAY; 
         check_desp( to_integer( unsigned(op) ), unsigned(entrada), 
                     unsigned(salida) , error_count );
      end loop; 
      -- Informe mostrando el resultado del test
      if ( error_count = 0 ) then
         report "Finaliza la simulacin: 0 errores";
      else
         report "Finaliza la simulacin: "  & 
                integer'image(error_count)  & " errores";
      end if;   
      wait;  -- Termina la simulacin
   end process vec_test;

end architecture bp_desplazador;