-- Banco de pruebas del restador, empleando una funcion
-- fichero: restadorCompleto1bit_bp_funcion.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity bp_rest_completo is     -- El banco de pruebas no tiene
end entity bp_rest_completo;   -- ni entradas ni salidas

architecture bp_rest_completo_funcion of bp_rest_completo is
   signal res, acarreo_out : std_logic;  -- Para conectar el UUT
   signal a, b, acarreo_in : std_logic;  -- Para conectar el UUT
  
   function error_check( esperado, actual : unsigned;
                         tnow             : time ) return integer is
   begin
      assert ( esperado = actual )       -- Comprobacion usando assert
         report "ERROR : Esperado ("     &   -- Report del error
            std_logic'image(esperado(1)) & 
            std_logic'image(esperado(0)) & 
            ") /= actual ("              &
            std_logic'image(actual(1))   & 
            std_logic'image(actual(0))   &
            ") en el instante"           & 
            time'image(tnow)
            severity error;                  -- Opcion severity
      if ( esperado /= actual ) then
         return 1;                      -- Indica error
      else 
         return 0;                      -- Indica que no hay error
      end if;
   end function error_check;
  
   component rest_completo is port
      ( res, acarreo_out : out std_logic;
        a, b, acarreo_in : in  std_logic );
   end component rest_completo;
begin
   uut : component rest_completo port map
         (res, acarreo_out, a, b, acarreo_in);
   -- Crear vectores de test y comprobar salidas del UUT
   gen_vec_test : process
      variable test_in     : unsigned (2 downto 0); -- Vector de test
      variable esperado    : unsigned (1 downto 0); -- Salida esperada
      variable num_errores : integer := 0;          -- Nm. errores
   begin
      test_in := B"000";
      for count in 0 to 7 loop
         a          <= test_in(2);
         b          <= test_in(1);
         acarreo_in <= test_in(0);
         wait for 10 ns;
         esperado   := ('0' & a) - ('0' & b) - ('0' & acarreo_in);
         num_errores := num_errores + 
                        error_check(esperado, acarreo_out & res, now);
         test_in := test_in + 1;
      end loop;
      report "Hay " & integer'image(num_errores) & " errores.";
      wait;
   end process gen_vec_test;
end architecture bp_rest_completo_funcion;