-- Banco de pruebas del sumador completo de 1 bit
-- fichero: sumadorCompleto1bit_bp.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity bp_sum_completo is     -- El banco de pruebas no tiene
end entity bp_sum_completo;   -- ni entradas ni salidas

architecture bp_sum_completo of bp_sum_completo is
  signal s, C_out, x, y, C_in  : std_logic;  -- Para conectar el UUT
  component sum_completo is 
     port ( s, C_out   : out std_logic;
            x, y, C_in : in  std_logic );
  end component sum_completo;
begin
  -- Instanciar y conectar UUT
  uut : component sum_completo port map
        (s => s, C_out => C_out, x => x, y => y, C_in => C_in);

  -- Crear vectores de test y comprobar salidas del UUT
  gen_vec_test : process
     variable test_in     : unsigned (2 downto 0); -- Vector de test
     variable esperado    : unsigned (1 downto 0); -- Salida esperada
     variable num_errores : integer := 0;          -- Nm. errores
  begin
     test_in := B"000";
     for count in 0 to 7 loop
        C_in   <= test_in(2);
        x      <= test_in(1);
        y      <= test_in(0);
        wait for 50 ns;
        esperado   := ('0' & x) + ('0' & y) + ('0' & C_in);
        if (esperado /= ( C_out & s )) then    -- Comprueba resultado
           report "ERROR : Esperado ("          & -- Report del error
                   std_logic'image(esperado(1)) &
                   std_logic'image(esperado(0)) & 
                   ") /= actual ("              &
                   std_logic'image(C_out)       &
                   std_logic'image(s)           &
                   ") en el instante"           &
                   time'image(now);
           num_errores := num_errores + 1;
        end if;
        test_in := test_in + 1;
     end loop;
     report "Test completo. Hay "      &
            integer'image(num_errores) & " errores.";
     wait;
  end process gen_vec_test;

end architecture bp_sum_completo;