-- Banco de pruebas del registro serie-a-paralelo y paralelo-a-serie 
-- de 4 bits
-- fichero: bp_registroSerieParalelo.vhd
library IEEE;
use IEEE.std_logic_1164.all;

entity bp_registroSerieParalelo is 
end entity bp_registroSerieParalelo;

architecture bp_registroSerieParalelo of bp_registroSerieParalelo is

   constant PERIODO  : time       := 200 ns;         -- Reloj

   signal Serial_out : std_logic;                    -- Salidas UUT
   signal Q          : std_logic_vector(3 downto 0); 
   signal Clock      : std_logic  :='0';             -- Entradas UUT
   signal SHSel      : std_logic_vector(1 downto 0);
   signal Serial_in  : std_logic;
   signal D          : std_logic_vector(3 downto 0);          
  
   component registroSerieParalelo is
      port ( Serial_out : out std_logic;
             Q          : out std_logic_vector(3 downto 0);
             Clock      : in  std_logic;
             SHSel      : in  std_logic_vector(1 downto 0);
             Serial_in  : in  std_logic;
             D          : in  std_logic_vector(3 downto 0)  );
   end component registroSerieParalelo;

begin
   -- Instanciar y conectar UUT
   uut : component registroSerieParalelo port map
         (Serial_out, Q, Clock, SHSel, Serial_in, D);
                  
   Clock <= not Clock after (PERIODO/2);             
                  
   gen_vec_test : process is
   begin
      report "Comienza la simulacin";
    
      -- Operacion Paralelo Serie --
      -- Realizamos una escritura en paralelo
      -- Despus vamos desplazando los bits que vamos
      -- leyendo de la salida serie
    
      SHSel     <= "01";  -- Carga paralelo
      Serial_in <= '0';
      D         <= "1010";
      wait until falling_edge(Clock);  -- 200 ns
    
      SHSel <= "10";      -- Desplaza a la drcha
      wait until falling_edge(Clock);  -- 400ns

      SHSel <= "10";      -- Desplaza a la drcha
      wait until falling_edge(Clock);  -- 600 ns

      SHSel <= "10";      -- Desplaza a la drcha
      wait until falling_edge(Clock);  -- 800 ns
    
      -- Operacion Serie Paralelo --
      -- Desplazamos los bits que van llegando por 
      -- la entrada serie.
      -- Despus, realizamos una lectura en paralelo

      SHSel <= "01"; Serial_in <= '0'; D <= "0000"; -- Carga paralelo
      wait until falling_edge(Clock);  -- 1000ns

      SHSel <= "10"; Serial_in <= '1'; -- Desplaza a la drcha
      wait until falling_edge(Clock);  -- 1200 ns
    
      SHSel <= "10"; Serial_in <= '0'; -- Desplaza a la drcha
      wait until falling_edge(Clock);  -- 1400 ns

      SHSel <= "10"; Serial_in <= '1'; -- Desplaza a la drcha
      wait until falling_edge(Clock);  -- 1600 ns
     
      SHSel <= "10"; Serial_in <= '0'; -- Desplaza a la drcha y leemos
      wait until falling_edge(Clock);  -- 1800 ns  

      report "Finaliza la simulacin";
      wait;     -- Final del bloque process

   end process gen_vec_test;

end architecture bp_registroSerieParalelo;