-- Banco de pruebas de la RAM 16x8
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity bp_RAM is
   constant PERIODO : time  := 100 ns; -- Retardo usado en el test
end entity bp_RAM;

architecture bp_RAM of bp_RAM is  
   signal wr_ena   : std_logic;
   signal clk      : std_logic  := '0';
   signal addr     : integer range 0 to 15;     
   signal data_in  : std_logic_vector (7 downto 0);
   signal data_out : std_logic_vector (7 downto 0);
   
   component ram is
      generic( bits  : integer := 8;      -- Nm. bits por palabra
               words : integer := 16 );   -- Nm. palabras en mem.
      port( data_out : out std_logic_vector (7 downto 0);
            wr_ena   : in  std_logic;
            clk      : in  std_logic;
            addr     : in  integer range 0 to 15;
            data_in  : in  std_logic_vector (7 downto 0) );
   end component ram;

begin  

   UUT : component ram port map
         (data_out, wr_ena, clk, addr, data_in);
        
   clk <= not clk after (PERIODO/2);   
  
   gen_vec_test : process is
   begin
      report "Comienza la simulacin";
  
      wr_ena <= '1';   -- Habilitada la escritura
      addr   <= 3; data_in <= "00000001"; wait until falling_edge(clk);
      addr   <= 4; data_in <= "00000010"; wait until falling_edge(clk);
      addr   <= 5; data_in <= "00000100"; wait until falling_edge(clk);

      wr_ena <= '0';   -- Deshabilitada la escritura
      addr   <= 3; data_in <= "00001000"; wait until falling_edge(clk);
      addr   <= 4; data_in <= "00010010"; wait until falling_edge(clk);
      addr   <= 5; data_in <= "00010100"; wait until falling_edge(clk); 
   
      report "Finaliza la simulacin";
      wait;       
   end process gen_vec_test;

end architecture bp_RAM;