-- Banco de pruebas del registro multifuncin
-- fichero: bp_Reg4mf.vhd
library IEEE;
use IEEE.std_logic_1164.all;

entity bp_Reg4mf is 
end entity bp_Reg4mf;

architecture bp_Reg4mf of bp_Reg4mf is
   constant PERIODO      : time       := 10 ns;   
   signal Q              : std_logic_vector(3 downto 0); 
   signal clk            : std_logic  := '0';            
   signal I              : std_logic_vector(3 downto 0);
   signal rst            : std_logic;        
   signal Shr_In, Shl_In : std_logic;
   signal Ld, Shr, Shl   : std_logic;   
  
   component Reg4mf is
      port ( Q              : out std_logic_vector(3 downto 0);
             I              : in  std_logic_vector(3 downto 0);
             Shr_In, Shl_In : in  std_logic;
             Ld, Shr, Shl   : in  std_logic;
             clk, rst       : in  std_logic  );
   end component Reg4mf;

begin
   -- Instanciar y conectar UUT
   uut : component Reg4mf port map 
         (Q, I, Shr_In, Shl_In, Ld, Shr, Shl, clk, rst);
                  
   clk <= not clk after (PERIODO/2);             
                  
   gen_vec_test : process is
      variable numTest : integer := 0;
   begin
      report "Comienza la simulacin";
    
      -- Reset
      I   <= "1111";  Shr_In <= '1'; Shl_In <= '1'; 
      rst <= '1'; Ld <= '1'; Shr <= '1'; Shl <= '1';
      wait until rising_edge(clk);
      wait for (PERIODO/2);
      numTest := numTest + 1;
      assert ( Q = "0000" ) 
         report "Error vector " & integer'image(numTest);
    
      -- Carga 1111
      I   <= "1111"; Shr_In <= '0'; Shl_In <= '0'; 
      rst <= '0'; Ld <= '1'; Shr <= '1'; Shl <= '1';
      wait until rising_edge(clk);
      wait for (PERIODO/2);
      numTest := numTest + 1;
      assert ( Q = "1111" ) 
         report "Error vector " & integer'image(numTest);

      -- Carga 1010
      I   <= "1010"; Shr_In <= '0'; Shl_In <= '1'; 
      rst <= '0'; Ld <= '1'; Shr <= '1'; Shl <= '1';
      wait until rising_edge(clk);
      wait for (PERIODO/2);
      numTest := numTest + 1;
      assert ( Q = "1010" ) 
         report "Error vector " & integer'image(numTest);

      -- Carga 0101
      I   <= "0101"; Shr_In <= '1'; Shl_In <= '1'; 
      rst <= '0'; Ld <= '1'; Shr <= '1'; Shl <= '1';
      wait until rising_edge(clk);
      wait for (PERIODO/2);
      numTest := numTest + 1;
      assert ( Q = "0101" ) 
         report "Error vector " & integer'image(numTest);
        
      -- Carga 0011
      I   <= "0011"; Shr_In <= '1'; Shl_In <= '1'; 
      rst <= '0'; Ld <= '1'; Shr <= '1'; Shl <= '1';
      wait until rising_edge(clk);
      wait for (PERIODO/2);
      numTest := numTest + 1;
      assert ( Q = "0011" ) 
         report "Error vector " & integer'image(numTest);

      -- Desplazamiento Derecha, Carga serie '1'
      I   <= "1111"; Shr_In <= '1'; Shl_In <= '1'; 
      rst <= '0'; Ld <= '0'; Shr <= '1'; Shl <= '0';
      wait until rising_edge(clk);
      wait for (PERIODO/2);
      numTest := numTest + 1;
      assert ( Q = "1001" ) 
         report "Error vector " & integer'image(numTest);

      -- Desplazamiento Derecha, Carga serie '0'
      I   <= "1111"; Shr_In <= '0'; Shl_In <= '1'; 
      rst <= '0'; Ld <= '0'; Shr <= '1'; Shl <= '0';
      wait until rising_edge(clk);
      wait for (PERIODO/2);
      numTest := numTest + 1;
      assert ( Q = "0100" ) 
         report "Error vector " & integer'image(numTest);

      -- Carga 0000
      I   <= "0000"; Shr_In <= '1'; Shl_In <= '1'; 
      rst <= '0'; Ld <= '1'; Shr <= '1'; Shl <= '1';
      wait until rising_edge(clk);
      wait for (PERIODO/2);
      numTest := numTest + 1;
      assert ( Q = "0000" ) 
         report "Error vector " & integer'image(numTest);

      -- Desplazamiento Izquierda, Carga serie '1'
      I   <= "1111"; Shr_In <= '1'; Shl_In <= '1'; 
      rst <= '0'; Ld <= '0'; Shr <= '0'; Shl <= '1';
      wait until rising_edge(clk);
      wait for (PERIODO/2);
      numTest := numTest + 1;
      assert ( Q = "0001" ) 
         report "Error vector " & integer'image(numTest);

      -- Desplazamiento Izquierda, Carga serie '0'
      I   <= "1111"; Shr_In <= '1'; Shl_In <= '0'; 
      rst <= '0'; Ld <= '0'; Shr <= '0'; Shl <= '1';
      wait until rising_edge(clk);
      wait for (PERIODO/2);
      numTest := numTest + 1;
      assert ( Q = "0010" ) 
         report "Error vector " & integer'image(numTest);

      -- Reset
      I   <= "1111";  Shr_In <= '1'; Shl_In <= '1'; 
      rst <= '1'; Ld <= '1'; Shr <= '1'; Shl <= '1';
      wait until rising_edge(clk);
      wait for (PERIODO/2);
      numTest := numTest + 1;
      assert ( Q = "0000" ) 
         report "Error vector " & integer'image(numTest);

      report "Finaliza la simulacin";
      wait;     -- Final del bloque process
   end process gen_vec_test;

end architecture bp_Reg4mf;