-- Register file 4x32. Descripcin del comportamiento
-- fichero: RegFile4x32_comp.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity RegFile4x32 is
   port ( R_data         : out std_logic_vector(31 downto 0);
          W_data         : in  std_logic_vector(31 downto 0);
          R_addr, W_addr : in  std_logic_vector( 1 downto 0);  
          R_en, W_en     : in  std_logic;
          clk, rst       : in  std_logic  ); 
end entity RegFile4x32;

architecture RegFile4x32_comp of RegFile4x32 is
    
   type regfile_type is
      array (0 to 3) of std_logic_vector(31 downto 0);
   signal regfile : regfile_type;   
   
begin
    
   W_process : process (clk)
   begin
      if rising_edge(clk) then
         if ( rst = '1' ) then
            regfile(0) <= X"00000000";
            regfile(1) <= X"00000000";
            regfile(2) <= X"00000000";
            regfile(3) <= X"00000000";
         elsif ( W_en = '1' ) then
            regfile(conv_integer(W_addr)) <= W_data;
         end if;
      end if;    
   end process W_process;
   
   R_process: process (R_addr, R_en, regfile)
   begin
      if ( R_en = '1' ) then
         R_data <= regfile( conv_integer(R_addr) );
      else 
         R_data <= (others => 'Z');
      end if;    
   end process R_process;
       
end architecture RegFile4x32_comp;