-- Banco de pruebas del register file
-- fichero: bp_RegFile4x32.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use std.textio.all;

entity bp_regFile4x32 is 
   constant PERIODO : time := 10 ns;
end entity bp_regFile4x32;

architecture bp_regFile4x32 of bp_regFile4x32 is  
   signal R_data         : std_logic_vector(31 downto 0);
   signal W_data         : std_logic_vector(31 downto 0);
   signal R_addr, W_addr : std_logic_vector( 1 downto 0);  
   signal R_en, W_en     : std_logic;
   signal rst            : std_logic;
   signal clk            : std_logic := '0';  
   
   component RegFile4x32 is
      port ( R_data         : out std_logic_vector(31 downto 0);
             W_data         : in  std_logic_vector(31 downto 0);
             R_addr, W_addr : in  std_logic_vector( 1 downto 0);  
             R_en, W_en     : in  std_logic;
             clk, rst       : in  std_logic );
   end component RegFile4x32;
begin
   uut : component RegFile4x32 port map 
         (R_data, W_data, R_addr, W_addr, R_en, W_en, clk, rst);
   clk <= not clk after (PERIODO/2);             
                  
   gen_vec_test : process is
      file     fichVectores : text;
      variable inputLinea   : line;
      variable inputBit     : bit;
      constant numBits      : integer  := 38;  -- Num bits menos 1 
      variable aux          : std_logic_vector(numBits downto 0);
   begin
      report "Comienza la simulacin";  
      -- Carga de vectores de test desde fichero
      file_open(fichVectores, "vectores.txt", read_mode);
      while not endfile(fichVectores) loop
         -- Lectura desde fichero 
         readline(fichVectores, inputLinea);
         for i in numBits downto 0 loop
            read(inputLinea, inputBit);
            if inputBit = '1' then
               aux(i) := '1';
            else
               aux(i) := '0';
            end if;
         end loop; -- bucle for
         -- Asigna valor a W_data, W_addr, W_en, R_addr, R_en
         rst    <= aux(38);
         W_data <= aux(37 downto 6);
         W_addr <= aux( 5 downto 4);
         W_en   <= aux( 3);
         R_addr <= aux( 2 downto 1);
         R_en   <= aux( 0);
         wait for PERIODO;
      end loop;    -- bucle while
      file_close(fichVectores);
      wait until rising_edge(clk);
      report "Finaliza la simulacin";
      wait;        -- Final del bloque process
   end process gen_vec_test;
end architecture bp_regFile4x32;