-- Conexin de la fuente de datos bidireccional
-- y la fuente unidireccional mediante un bus.
-- fichero: bus.vhd
library IEEE; 
use IEEE.std_logic_1164.all;

entity tb_bus is 
   constant WORD_SZ : integer  := 16;      -- Bits por palabra
   constant PERIOD  : time     := 100 ns;  -- Periodo ciclo reloj
end entity tb_bus;

architecture tb_bus of tb_bus is
   signal data      : std_logic_vector(WORD_SZ-1 downto 0); 
                    -- Datos en el bus  
   signal OE1_n     : std_logic;  
                    -- Habilita lectura memoria lectura/escritura
   signal OE2_n     : std_logic;  
                    -- Habilita lectura memoria slo lectura 
   signal WE_n      : std_logic;  
                    -- Habilita escritura 
   signal writeData : std_logic_vector(WORD_SZ-1 downto 0);  
                    -- Datos a escribir  

   component fuenteBidireccional is 
      generic ( WORD_SIZE   : integer := 8;      -- Bits por palabra
                READ_DELAY  : time    := 10 ns;  -- Retardo lectura
                WRITE_DELAY : time    := 10 ns); -- Retardo escritura
      port ( data : inout std_logic_vector(WORD_SIZE-1 downto 0);  
                                       -- Seal de datos bidireccional
             OE_n : in    std_logic;   -- Habilita lectura
             WE_n : in    std_logic ); -- Habilita escritura
   end component fuenteBidireccional;

   component fuenteUnidireccional is 
      generic ( WORD_SIZE  :  integer := 8;      -- Bits por palabra
                READ_DELAY :  time    := 10 ns); -- Retardo en lectura
      port ( data : out std_logic_vector(WORD_SIZE-1 downto 0);  
                                        -- Seal de datos bidireccional
             OE_n : in    std_logic);   -- Habilita lectura
   end component fuenteUnidireccional;

begin
   -- Instanciar fuentes de datos, asignando valor a constantes generic
   U1 : component fuenteBidireccional
        generic map (WORD_SZ, 30 ns, 40 ns) -- Asigna const. generic
        port    map (data, OE1_n, WE_n);
   U2 : component fuenteUnidireccional
        generic map (WORD_SZ, 20 ns )       -- Asigna const. generic
        port    map (data, OE2_n);

   data <= writeData when ( WE_n = '0' ) else ( others => 'Z' );  

   vectoresTest : process is
   begin
      OE1_n <= '1'; OE2_n <= '1'; WE_n <= '1'; -- Deshabilita seales 
      writeData <= B"0000_1111_1100_0011";     -- (=0x0fc3)
      wait for PERIOD;
      WE_n <= '0';      -- Habilita escritura (se escribe 0x0fc3)
      wait for PERIOD;
      WE_n  <= '1';     -- Deshabilita escritura
      OE2_n <= '0';     -- Habilita lectura desde fuente unidireccional
                        -- El dato en el bus debe ser 0x0001
      wait for PERIOD;
      OE2_n <= '1';     -- Deshabilita lectura de fuente unidireccional
      OE1_n <= '0';     -- Habilita lectura desde fuente bidireccional
                        -- El dato en el bus debe ser 0x0fc3
      wait for PERIOD;
      wait;        
   end process vectoresTest;

end architecture tb_bus;