-- Fuente de datos bidireccional (lectura y escritura)
-- fichero: fuenteBidireccional.vhd
library IEEE; 
use IEEE.std_logic_1164.all;

entity fuenteBidireccional is 
   generic ( WORD_SIZE   : integer := 8;      -- Bits por palabra, 
                                              -- por defecto 8 bits
             READ_DELAY  : time    := 10 ns;  -- Retardo en lectura, 
                                              -- por defecto 10 ns
             WRITE_DELAY : time    := 10 ns); -- Retardo en escritura, 
                                              -- por defecto 10 ns
   port ( data : inout std_logic_vector(WORD_SIZE-1 downto 0);  
                                     -- Seal de datos bidireccional
          OE_n : in    std_logic;    -- Habilita lectura
          WE_n : in    std_logic );  -- Habilita escritura
end entity fuenteBidireccional; 

architecture fuenteBidireccional of fuenteBidireccional is
   signal data_stored : std_logic_vector(WORD_SIZE-1 downto 0); 
                                     -- Datos almacenados internamente
begin
   ram : process (OE_n, WE_n, data)
   begin
      if (OE_n = '0') then
         data <= data_stored after READ_DELAY;
      elsif ( WE_n = '0') then
         data_stored <= data after WRITE_DELAY;
      else 
         data <= (others => 'Z');              
      end if;
   end process ram;
end architecture fuenteBidireccional;