-- Banco de pruebas de la latch D
-- fichero: bp_latch_D.vhd
library IEEE;
use IEEE.std_logic_1164.all;

entity bp_latch_D is 
end entity bp_latch_D;

architecture bp_latch_D of bp_latch_D is
   signal   Q         : std_logic;     -- Salidas UUT
   signal   D, Enable : std_logic;     -- Entradas UUT
  
   component latch_D is
      port ( Q          : out std_logic;
             D, Enable  : in std_logic  );
   end component latch_D;

   -- Procedimiento para comprobar las salidas de la latch
   procedure comprueba_salidas
      ( esperado_Q            :       std_logic;
        actual_Q              :       std_logic;
        error_count           : inout integer) is
   begin
      -- Comprueba Q
      if ( esperado_Q /= actual_Q ) then
         report "ERROR: Estado esperado ("     &
                std_logic'image(esperado_Q)    &
                "), estado actual ("           &
                std_logic'image(actual_Q)      &
                "), instante: "                &
                time'image(now);
         error_count := error_count + 1;
      end if;
   end procedure comprueba_salidas;

begin
   uut : component latch_D port map  -- Instanciar y conectar UUT
         (Q, D, Enable);
                  
   gen_vec_test : process is
      variable error_count : integer := 0; -- Nm. errores
   begin
      report "Comienza la simulacin";
      -- Vectores de test y comprobacin del resultado
      D <= '0'; Enable <= '0';   wait for 10ns;   -- 10ns
      Enable <= '1';             wait for  5ns;   -- 15ns
      comprueba_salidas('0',Q,error_count);  
      D <= '1'; Enable <= '0';   wait for 10ns;   -- 25ns
      Enable <= '1';             wait for  5ns;   -- 30ns
      comprueba_salidas('1',Q,error_count); 
      Enable <= '0'; 
            
      -- Informe final
      if (error_count = 0) then
         report "Simulacin finalizada sin errores";
      else
         report "ERROR: Hay "              & 
                integer'image(error_count) &
                " errores.";
      end if;
      wait;   
   end process gen_vec_test;
end architecture bp_latch_D;