-- Banco de pruebas del circuito detector de secuencia
-- fichero: bp_detector.vhd
library IEEE;
use IEEE.std_logic_1164.all;

entity bp_detector is 
end entity bp_detector;

architecture bp_detector of bp_detector is
   constant PERIODO   : time       := 100 ns; -- Reloj
   signal   state     : std_logic_vector(1 downto 0); -- Salidas UUT
   signal   Y         : std_logic;
   signal   clk       : std_logic  := '0';            -- Entradas UUT
   signal   reset, X  : std_logic;           
  
   component detector is
      port( Y     : out std_logic;
            state : out std_logic_vector(1 downto 0);
            X     : in  std_logic;
            reset : in  std_logic;
            clk   : in  std_logic );
   end component detector;

   -- Procedimiento para comprobar las salidas 
   procedure comprueba_salidas
      ( esperado_state :       std_logic_vector(1 downto 0);
        actual_state   :       std_logic_vector(1 downto 0);
        esperado_Y     :       std_logic;
        actual_Y       :       std_logic;
        error_count    : inout integer) is
   begin
      -- Comprueba state
      if ( esperado_state /= actual_state ) then
         report "ERROR: Estado esperado ("            &
                std_logic'image(esperado_state(1))    &
                std_logic'image(esperado_state(0))    & 
                "), estado actual ("                  &
                std_logic'image(actual_state(1))      &
                std_logic'image(actual_state(0))      &
                "), instante: "                       &
                time'image(now);
         error_count := error_count + 1;
      end if;

      -- Comprueba Y
      if ( esperado_Y /= actual_Y ) then
         report "ERROR: Salida Y esperada ("   &
                std_logic'image(esperado_Y)    &
                "), salida actual ("           &
                std_logic'image(actual_Y)      &
                "), instante: "                &
                time'image(now);
         error_count := error_count + 1;
      end if;
   end procedure comprueba_salidas;

begin
   
   uut : component detector port map
         (Y, state, X, reset, clk);
                  
   reset <= '0', 
            '1' after (PERIODO/4),
            '0' after (PERIODO + PERIODO/4);

   clk <= not clk after (PERIODO/2); 
                 
   gen_vec_test : process is
    variable error_count : integer := 0;        -- Nm. errores
   begin
      report "Comienza la simulacin";
      -- Vectores de test y comprobacin del resultado
      X <= '0';   wait for PERIODO;                        --  1
      comprueba_salidas("00", state, '0', Y, error_count); 
      X <= '1';   wait for PERIODO;                        --  2
      comprueba_salidas("01", state, '0', Y, error_count);   
      X <= '1';   wait for PERIODO;                        --  3
      comprueba_salidas("10", state, '0', Y, error_count); 
      X <= '1';   wait for PERIODO;                        --  4
      comprueba_salidas("11", state, '1', Y, error_count); 
      X <= '1';   wait for PERIODO;                        --  5
      comprueba_salidas("11" ,state, '1', Y, error_count); 
      X <= '0';   wait for PERIODO;                        --  6
      comprueba_salidas("00", state, '0', Y, error_count); 
      X <= '1';   wait for PERIODO;                        --  7
      comprueba_salidas("01", state, '0', Y, error_count); 
      X <= '0';   wait for PERIODO;                        --  8
      comprueba_salidas("00", state, '0', Y, error_count);     
      X <= '1';   wait for PERIODO;                        --  9
      comprueba_salidas("01", state, '0', Y, error_count); 
      X <= '1';   wait for PERIODO;                        -- 10
      comprueba_salidas("10", state, '0', Y, error_count); 
      X <= '0';   wait for PERIODO;                        -- 11
      comprueba_salidas("00", state, '0', Y, error_count);       
      -- Informe final
      report "Hay "                      & 
             integer'image(error_count ) &
             " errores.";
      wait; 
   end process gen_vec_test;

end architecture bp_detector;
