-- Banco de pruebas del contador BCD
-- fichero: bp_contadorBCD.vhd
library IEEE;
use IEEE.std_logic_1164.all;

entity bp_contador is 
end entity bp_contador;

architecture bp_contador of bp_contador is
   constant PERIODO : time       := 100 ns;         -- Reloj
   signal   q       : std_logic_vector(3 downto 0); -- Salidas UUT
   signal   clk     : std_logic  := '0';            -- Entradas UUT
   signal   reset   : std_logic;           
  
   component contador is
      port ( count      : out std_logic_vector(3 downto 0);
             clk, reset : in std_logic );
   end component contador;

   -- Procedimiento para comprobar las salidas 
   procedure comprueba_salidas
      ( esperado_q   :       std_logic_vector(3 downto 0);
        actual_q     :       std_logic_vector(3 downto 0);
        error_count  : inout integer ) is
   begin
      -- Comprueba q
      if ( esperado_q /= actual_q ) then
         report "ERROR: Estado esperado ("        &
                std_logic'image(esperado_q(3))    &
                std_logic'image(esperado_q(2))    &           
                std_logic'image(esperado_q(1))    &
                std_logic'image(esperado_q(0))    & 
                "), estado actual ("              &
                std_logic'image(actual_q(3))      &
                std_logic'image(actual_q(2))      &                 
                std_logic'image(actual_q(1))      &
                std_logic'image(actual_q(0))      &
                "), instante: "                   &
                time'image(now);
         error_count := error_count + 1;
      end if;
   end procedure comprueba_salidas;

begin
   
   uut : component contador port map
         (q,  clk, reset);
                  
   reset <= '0', 
            '1' after (PERIODO/4),
            '0' after (PERIODO + PERIODO/4);

   clk <= not clk after (PERIODO/2);      
                     
   gen_vec_test : process is
      variable error_count : integer := 0; -- Nm. errores
   begin
      report "Comienza la simulacin";

      -- Vectores de test y comprobacin del resultado
      wait for PERIODO;                           -- 1
      comprueba_salidas("0000", q, error_count); 
      wait for PERIODO;                           -- 2
      comprueba_salidas("0001", q, error_count);  
      wait for PERIODO;                           -- 3
      comprueba_salidas("0010", q, error_count); 
      wait for PERIODO;                           -- 4
      comprueba_salidas("0011", q, error_count); 
      wait for PERIODO;                           -- 5
      comprueba_salidas("0100" ,q, error_count); 
      wait for PERIODO;                           -- 6
      comprueba_salidas("0101", q, error_count); 
      wait for PERIODO;                           -- 7
      comprueba_salidas("0110", q, error_count); 
      wait for PERIODO;                           -- 8
      comprueba_salidas("0111", q, error_count);  
      wait for PERIODO;                           -- 9
      comprueba_salidas("1000", q, error_count); 
      wait for PERIODO;                           -- 10
      comprueba_salidas("1001", q, error_count);         
      wait for PERIODO;                           -- 11
      comprueba_salidas("0000", q, error_count);       

      -- Informe final
        report  "ERROR: Hay "               & 
                integer'image(error_count ) &
                " errores.";
      wait;      
   end process gen_vec_test;

end architecture bp_contador;