-- Banco de pruebas del contador binario de 4 bits
-- fichero: bp_udcounter.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity bp_udcounter is 
end entity bp_udcounter;

architecture bp_udcounter of bp_udcounter is
   constant PERIODO   : time       := 10 ns;          -- Reloj
   signal   Q         : std_logic_vector(3 downto 0); -- Salidas UUT
   signal   Clock     : std_logic  := '0';            -- Entradas UUT
   signal   Clear, Count, Down  : std_logic;           
  
   component udcounter is
      port ( Q :  out std_logic_vector(3 downto 0);
             Clock, Clear, Count, Down :  in std_logic );
   end component udcounter;

   -- Procedimiento para comprobar las salidas 
   procedure comprueba_salidas
      ( esperado_q      :       std_logic_vector(3 downto 0);
        actual_q        :       std_logic_vector(3 downto 0);
        error_count     : inout integer) is
   begin
      -- Comprueba q
      if ( esperado_q /= actual_q ) then
         report "ERROR: Estado esperado ("        &
                std_logic'image(esperado_q(3))    &
                std_logic'image(esperado_q(2))    &           
                std_logic'image(esperado_q(1))    &
                std_logic'image(esperado_q(0))    & 
                "), estado actual ("              &
                std_logic'image(actual_q(3))      &
                std_logic'image(actual_q(2))      &                 
                std_logic'image(actual_q(1))      &
                std_logic'image(actual_q(0))      &
                "), instante: "                   &
                time'image(now);
         error_count := error_count + 1;
      end if;
   end procedure comprueba_salidas;

begin
   
   uut : component udcounter port map
         (Q, Clock, Clear, Count, Down);
                  
   Clear <= '0', 
            '1' after (PERIODO/4),
            '0' after (3*PERIODO/4);

   Clock <= not Clock after (PERIODO/2);                           

   gen_vec_test : process is
      variable temp        : unsigned (3 downto 0);  
      variable error_count : integer  := 0;    -- Nm. errores
   begin
      report "Comienza la simulacin";

      -- Vectores de test y comprobacin del resultado
      wait for PERIODO;
      Count <= '1'; Down <= '0';          
      for i in 0 to 4 loop
         temp := TO_UNSIGNED(i,4);
         comprueba_salidas( std_logic_vector(temp), Q, error_count);      
         wait for PERIODO;     
      end loop;   
      Count <= '0'; Down <= '1';   
      wait for PERIODO;   
      Count <= '1'; 
      for i in 5 downto 1 loop
         temp := TO_UNSIGNED(i,4);
         comprueba_salidas( std_logic_vector(temp), Q, error_count);
         wait for PERIODO;                 
      end loop;    
      Count <= '0'; 

      -- Informe final
      if ( error_count = 0 ) then
         report "Simulacin finalizada sin errores";
      else
         report "ERROR: Hay "               & 
                integer'image(error_count ) &
                " errores.";
      end if;
      wait;     -- Final del bloque process
   end process gen_vec_test;

end architecture bp_udcounter;
