-- Banco de pruebas del controlador de dos semforos
-- fichero: bp_ctrl_semaforo.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use work.ctrl_semaforo_CONSTANTS.all;

entity bp_ctrl_semaforo is 
end entity bp_ctrl_semaforo;

architecture bp_ctrl_semaforo of bp_ctrl_semaforo is
   signal   z     : std_logic_vector(5 downto 0);  -- Salidas UUT
   signal   clk   : std_logic  := '0';             -- Entradas UUT
   signal   reset : std_logic;    
   
   component ctrl_semaforo is
      port ( r1, r2, y1, y2, g1, g2  : out std_logic;
             clk, reset              : in std_logic );
   end component ctrl_semaforo;

   procedure comprueba_salidas
      ( esperado_z    :       std_logic_vector(5 downto 0);
        actual_z      :       std_logic_vector(5 downto 0);
        error_count   : inout integer ) is
   begin
      if ( esperado_z /= actual_z ) then
         report "ERROR: Estado esperado ("        &
                std_logic'image(esperado_z(5))    &
                std_logic'image(esperado_z(4))    & 
                std_logic'image(esperado_z(3))    &
                std_logic'image(esperado_z(2))    &  
                std_logic'image(esperado_z(1))    &
                std_logic'image(esperado_z(0))    & 
                "), estado actual ("              &
                std_logic'image(actual_z(5))      &
                std_logic'image(actual_z(4))      & 
                std_logic'image(actual_z(3))      &
                std_logic'image(actual_z(2))      &  
                std_logic'image(actual_z(1))      &
                std_logic'image(actual_z(0))      & 
                "), instante: "                   &
               time'image(now);
         error_count := error_count + 1;
      end if;
   end procedure comprueba_salidas;

begin
  
   uut : component ctrl_semaforo port map
         (z(5), z(4), z(3), z(2), z(1), z(0), clk, reset);
             
   reset <= '0',
            '1' after (PERIODO/4),
            '0' after (PERIODO + PERIODO/4);

   clk <= not clk after (PERIODO/2);             
                  
   gen_vec_test : process is
      variable error_count : integer := 0; -- Nm. errores
   begin
      report "Comienza la simulacin";

      -- Vectores de test y comprobacin del resultado
      wait for PERIODO;                            -- 1
      comprueba_salidas("001100", z, error_count); 
      wait for PERIODO;                            -- 2
      comprueba_salidas("100100", z, error_count);  
      wait for timeRY*PERIODO;                     -- 3
      comprueba_salidas("010010", z, error_count); 
      wait for timeGR*PERIODO;                     -- 4
      comprueba_salidas("011000", z, error_count); 
      wait for timeYR*PERIODO;                     -- 5
      comprueba_salidas("100001", z, error_count); 
      wait for timeRG*PERIODO;                     -- 6
      comprueba_salidas("100100", z, error_count); 
      
      -- Informe final
      if (error_count = 0) then
         report "Simulacin finalizada sin errores";
      else
         report  "ERROR: Hay "              & 
                 integer'image(error_count) &
                 " errores.";
      end if;

      wait;  -- Finaliza el bloque process
   end process gen_vec_test;

end architecture bp_ctrl_semaforo;