-- Controlador de semforos
-- fichero: ctrl_semaforo.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use work.ctrl_semaforo_CONSTANTS.all;

entity ctrl_semaforo is
   port( r1, r2, y1, y2, g1, g2 : out std_logic;
         clk, reset             : in  std_logic );
end entity ctrl_semaforo;

architecture ctrl_semaforo of ctrl_semaforo is
   type type_state is (RG, RY, GR, YR, YY);
   signal internal_state : type_state;
begin

   proximo_estado : process (clk, reset)
      variable count : integer range 0 to timeMAX;
   begin
      if (reset = '1') then
         internal_state <= YY;
         count := 0;
      elsif (rising_edge(clk)) then
         case internal_state is
            when RG =>
               if (count = timeRG) then
                  internal_state <= RY;
                  count := 0;
               end if;
            when RY =>
               if (count = timeRY) then
                  internal_state <= GR;
                  count := 0;
               end if;
            when GR =>
               if (count = timeGR) then
                  internal_state <= YR;
                  count := 0;
               end if;
            when YR =>
               if (count = timeYR) then
                  internal_state <= RG;
                  count := 0;
               end if;
            when YY =>
               internal_state <= RY;
         end case;
         count := count + 1;              
      end if;
   end process proximo_estado;

   salidas : process(internal_state)
   begin
      case internal_state is
         when RG =>
            r1 <= '1'; r2 <= '0'; 
            y1 <= '0'; y2 <= '0'; 
            g1 <= '0'; g2 <= '1';
         when RY =>
            r1 <= '1'; r2 <= '0'; 
            y1 <= '0'; y2 <= '1'; 
            g1 <= '0'; g2 <= '0';
         when GR => 
            r1 <= '0'; r2 <= '1'; 
            y1 <= '0'; y2 <= '0'; 
            g1 <= '1'; g2 <= '0';
         when YR =>
            r1 <= '0'; r2 <= '1'; 
            y1 <= '1'; y2 <= '0'; 
            g1 <= '0'; g2 <= '0';
         when YY =>
            r1 <= '0'; r2 <= '0'; 
            y1 <= '1'; y2 <= '1'; 
            g1 <= '0'; g2 <= '0';
      end case;
   end process salidas;

end architecture ctrl_semaforo;

