-- Banco de pruebas del controlador del microondas
-- fichero: bp_microondas.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.microondas_CONST.all;

entity bp_microondas is 
end entity bp_microondas;

architecture bp_microondas of bp_microondas is
   -- Salidas UUT
   signal   calentar, luz, alarma : std_logic;         
   signal   segundos              : std_logic_vector(9 downto 0);  
   -- Entradas UUT
   signal   clk                   : std_logic := '0'; 
   signal   reset, minuto, marcha : std_logic;   
   signal   stop_Reset, puerta    : std_logic; 
  
   component microondas is
      port( segundos                : out std_logic_vector(9 downto 0);
            calentar, luz, alarma   : out std_logic;
            reset, minuto, marcha   : in  std_logic;
            stop_Reset, puerta, clk : in std_logic  );
   end component microondas;
 
   -- Procedimiento para comprobar las salidas 
   procedure comprueba 
      ( esperado_segundos  : integer;
        esperado_luz, esperado_alarma, esperado_calentar : std_logic;
        actual_segundos    : std_logic_vector(9 downto 0);
        actual_luz, actual_alarma, actual_calentar       : std_logic;
        error_count        : inout integer ) is   
      variable e_seg: integer  RANGE 0 TO 1023;
   begin
      if ( esperado_luz /= actual_luz ) then           -- Comprueba 
                                                       -- luz
         report "ERROR: Luz esperada ("            & 
                std_logic'image(esperado_luz)      &
                "), luz actual ("                  & 
                std_logic'image(actual_luz)        &
                "), instante: "                    &  
                time'image(now);
         error_count := error_count + 1;
      end if;
      if ( esperado_alarma /= actual_alarma ) then     -- Comprueba 
                                                       -- alarma
         report "ERROR: Alarma esperada ("         & 
                std_logic'image(esperado_alarma)   &
                "), alarma actual ("               & 
                std_logic'image(actual_alarma)     &
                "), instante: "                    & 
                time'image(now);
         error_count := error_count + 1;
      end if;
      if ( esperado_calentar /= actual_calentar ) then -- Comprueba 
                                                       -- calentar
         report "ERROR: Calentar esperado ("        & 
                std_logic'image(esperado_calentar)  &
                "), calentar actual ("              &
                std_logic'image(actual_calentar)    &
                "), instante: "                     & 
                time'image(now);
         error_count := error_count + 1;
      end if;   
      e_seg := to_integer( unsigned(actual_segundos) );
      if ( esperado_segundos /= e_seg ) then           -- Comprueba 
                                                       -- segundos
         report "ERROR: Segundos esperados ("        & 
                integer'image(esperado_segundos)     &
                "), segundos actual ("               & 
                integer'image(e_seg)                 &
                "), instante: "                      & 
                time'image(now);
         error_count := error_count + 1;
      end if;              
   end procedure comprueba;

begin
   
   uut : component microondas port map 
         ( segundos, calentar, luz, alarma, reset,
           minuto, marcha, stop_Reset, puerta, clk );

   reset <= '0', 
            '1' after (PERIODO/4), 
            '0' after (PERIODO + PERIODO/4);

   clk <= not clk after (PERIODO/2);   
                         
   gen_vec_test : process is
      variable error_count : integer := 0; -- Nm. errores
   begin
      report "Comienza la simulacin";
    
      -- Vectores de test y comprobacin del resultado
      minuto <= '0'; marcha <= '0'; stop_Reset <= '0'; puerta <= '0'; 
      wait for 2*PERIODO; 
      comprueba(0, '0', '0', '0',
                segundos, luz, alarma, calentar, error_count); 

      minuto <= '1'; marcha <= '1'; 
      wait for PERIODO;     
      comprueba(60, '0', '0', '0',
                segundos, luz, alarma, calentar, error_count);

      minuto <= '0'; 
      wait for PERIODO;                          -- 1 coccion
      comprueba(60, '1', '0', '1',
                segundos, luz, alarma, calentar, error_count); 

      minuto <= '1'; 
      wait for PERIODO;  
      minuto <= '0'; marcha <= '0'; 
      wait for PERIODO; 
      comprueba(119, '1', '0', '1',
                segundos, luz, alarma, calentar, error_count);

      minuto <= '0'; marcha <= '0'; 
      wait for 2*sesentasec*PERIODO;             -- 2 alarmaActivada
      comprueba(0, '0', '1', '0',
                segundos, luz, alarma, calentar, error_count);
 
      puerta <= '1'; 
      wait for PERIODO;                          -- 3 puertaAbierta
      comprueba(0, '1', '0', '0',
                segundos, luz, alarma, calentar, error_count);
 
      minuto <= '1';  
      wait for PERIODO;  
      comprueba(60, '1', '0', '0',
                segundos, luz, alarma, calentar, error_count); 

      minuto<='0'; stop_Reset<= '1'; 
      wait for 2*PERIODO;                        -- 4 puertaAbierta
      comprueba(0, '1', '0', '0',
                segundos, luz, alarma, calentar, error_count); 

      stop_Reset <= '0'; puerta <= '0'; 
      wait for PERIODO;                          -- 5 puertaCerrada
      comprueba(0, '0', '0', '0',
                segundos, luz, alarma, calentar, error_count); 

      minuto <= '1'; marcha <='1'; 
      wait for 2*PERIODO;                        -- 6 coccion
      comprueba(60, '1', '0', '1',
                segundos, luz, alarma, calentar, error_count); 

      minuto <= '0'; puerta <= '1'; marcha <= '0'; 
      wait for PERIODO;                          -- 7 puertaAbierta
      comprueba(59, '1', '0', '0',
                segundos, luz, alarma, calentar, error_count); 

      puerta <= '0'; 
      wait for PERIODO;                          -- 8 puertaCerrada
      comprueba(59, '0', '0', '0',
                segundos, luz, alarma, calentar, error_count); 

      puerta <= '1';   
      wait for PERIODO;                          -- 9 puertaAbierta
      comprueba(59, '1', '0', '0',
                segundos, luz, alarma, calentar, error_count); 

      puerta <= '0';   
      wait for PERIODO;                          -- 10 puertaCerrada
      comprueba(59, '0', '0', '0',
                segundos, luz, alarma, calentar, error_count); 

      marcha <= '1'; 
      wait for PERIODO;                          -- 11 coccion
      comprueba(59, '1', '0', '1',
                segundos, luz, alarma, calentar, error_count); 

      stop_Reset <= '1'; puerta <= '0'; 
      wait for PERIODO;                          -- 12 puertaCerrada 
      comprueba(59, '0', '0', '0',
                segundos, luz, alarma, calentar, error_count);

      marcha <= '1'; stop_Reset <= '0'; 
      wait for PERIODO;                          -- 13 coccion
      comprueba(59, '1', '0', '1',
                segundos, luz, alarma, calentar, error_count);

      wait for sesentasec*PERIODO;               -- 14 alarmaActivada
      comprueba(0, '0', '1', '0',
                segundos, luz, alarma, calentar, error_count); 

      wait for (ALARMAT-10)*PERIODO;                                
      comprueba(0, '0', '1', '0',
                segundos, luz, alarma, calentar, error_count); 

      minuto <= '1'; marcha <= '0'; 
      wait for PERIODO;             
      comprueba(60, '0', '1', '0',
                segundos, luz, alarma, calentar, error_count);

      stop_Reset <= '1'; minuto <= '0'; 
      wait for 2*PERIODO;                        -- 15 alarmaActivada 
      comprueba(0, '0', '1', '0',
                segundos, luz, alarma, calentar, error_count);

      stop_Reset <= '0'; 
      wait for 7*PERIODO;                        -- 16 puertaCerrada
      comprueba(0, '0', '0', '0',
                segundos, luz, alarma, calentar, error_count);

      minuto <= '1'; marcha <= '0';   
      wait for PERIODO;   
      comprueba(60, '0', '0', '0',
                segundos, luz, alarma, calentar, error_count); 

      stop_Reset <= '1'; 
      wait for 2*PERIODO;                        -- 17 puertaCerrada
      comprueba(0, '0', '0', '0',
                segundos, luz, alarma, calentar, error_count); 

      minuto <= '0'; stop_Reset <= '0';
    
      report "Finaliza la simulacin con " &     -- Informe final
             integer'image(error_count)    &
             "errores.";
      wait;     -- Final del bloque process
   end process gen_vec_test;

end architecture bp_microondas;