-- Circuito controlador del microondas
-- fichero: microondas.vhd
library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.numeric_std.all;
use work.microondas_CONST.all;

entity microondas is
   port ( segundos                : out std_logic_vector(9 downto 0); 
          calentar, luz, alarma   : out std_logic;
          reset, minuto, marcha, 
          stop_Reset, puerta, clk : in  std_logic );
end entity microondas;

architecture microondas of microondas is
   signal minutoAnterior :  std_logic;    
   signal internal_state :  estadoMicro;
   signal t_coccion      :  integer range 0 to TMAX;
   signal t_alarma       :  integer range 0 to ALARMAT;
   signal reset_alarma   :  boolean;
   signal resetTimer     :  boolean; 
begin 

   proximo_estado : process(clk, reset) is
   begin
      if (reset = '1') then
         internal_state <= puertaCerrada; 
         reset_alarma   <= true; 
         resetTimer     <= true;
      elsif rising_edge(clk) then
         case internal_state is
            when puertaAbierta =>
               if (puerta = '0') then 
                  internal_state <= puertaCerrada; 
               end if;
               if (stop_Reset = '1') then 
                  resetTimer <= true; 
               else 
                  resetTimer <= false;
               end if;
            when puertaCerrada =>
               if (puerta = '1') then
                  internal_state <= puertaAbierta;
               end if;
               if  (t_coccion > 0 and marcha = '1') then
                  internal_state <= coccion;
               end if;   
               if (stop_Reset = '1') then 
                  resetTimer <= true; 
               else 
                  resetTimer <= false;
               end if; 
            when coccion =>
               if (puerta = '1') then
                  internal_state <= puertaAbierta;
               end if;
               if  (t_coccion = 0) then
                  internal_state <= alarmaActivada;
                  resetTimer     <= true; 
                  reset_alarma   <= false;
               end if;           
               if (stop_Reset = '1') then 
                  internal_state <= puertaCerrada; 
               end if;
            when alarmaActivada =>
               if (puerta = '1') then
                  internal_state <= puertaAbierta;
                  reset_alarma   <= true;
               end if;
               if  (t_alarma = (ALARMAT - 1)) then
                  reset_alarma   <= true;
                  internal_state <= puertaCerrada;
               end if;
               if (stop_Reset = '1') then 
                  resetTimer <= true; 
               else 
                  resetTimer <= false;
               end if;
         end case;  
      end if;            
   end process proximo_estado;
   
   salidas : process (internal_state) is
   begin
      case internal_state is   
         when puertaAbierta =>
            calentar <= '0'; luz <= '1'; alarma <= '0';
         when puertaCerrada =>
            calentar <= '0'; luz <= '0'; alarma <= '0';
         when coccion =>
            calentar <= '1'; luz <= '1'; alarma <= '0'; 
         when alarmaActivada =>
            calentar <= '0'; luz <= '0'; alarma <= '1';
      end case;
   end process salidas;       

   --Convierte los segundos a std_logic_vector de longitud 10
   segundos <= std_logic_vector(to_unsigned((t_coccion+1)/unsec,10));

   contador: process (clk) is    -- Modifica t_coccion, t_alarma
   begin 
      if (reset_alarma) then
         t_alarma <= 0;
      elsif (internal_state = alarmaActivada and 
             t_alarma < ALARMAT                  ) then
         if (rising_edge(clk)) then 
            t_alarma <= t_alarma + 1;
         end if;
      end if;
      if (resetTimer) then
         t_coccion <= 0;
      elsif (minuto = '1' and minutoAnterior = '0') then
         t_coccion <= t_coccion + (sesentasec-1);
      elsif (internal_state = coccion and t_coccion > 0) then
         if (rising_edge(clk)) then
            t_coccion <= t_coccion - 1;               
         end if;
      end if;
      minutoAnterior <= minuto;
   end process contador;

end architecture microondas;