-- Banco de pruebas del flip-flop R-S
-- fichero: bp_flipflopRS.vhd
library IEEE;
use IEEE.std_logic_1164.all;

entity bp_flipflopRS is
end entity bp_flipflopRS;

architecture bp_flipflopRS of bp_flipflopRS is
   constant PERIODO       : time      := 100 ns;      -- Reloj
   signal   q, notq       : std_logic;          -- Salidas UUT
   signal   clk           : std_logic := '0';  -- Entradas UUT
   signal   R, S, reset_n : std_logic;

   component flipflopRS is
      port ( q, notq            : out std_logic;
             clk, R, S, reset_n : in  std_logic );
   end component flipflopRS;

   -- Procedimiento para comprobar las salidas del flip-flop
   procedure comprueba_salidas
      ( esperado_q           :       std_logic;
        actual_q, actual_q_n :       std_logic;
        error_count          : inout integer    ) is
   begin
      -- Comprueba q
      if ( esperado_q /= actual_q ) then
         report "ERROR: Estado esperado ("     &
                std_logic'image(esperado_q)    &
                "), estado actual ("           & 
                std_logic'image(actual_q)      &
                "), instante: "                &
                time'image(now);
         error_count := error_count + 1;
      end if;
      -- Comprueba q_n
      if ( (not esperado_q) /= actual_q_n ) then
         report "ERROR: q_n esperado ("            &
                std_logic'image((not esperado_q))  &
                "), valor actual ("                &
                std_logic'image(actual_q_n)        &
                "), instante: "                    &
                time'image(now);
         error_count := error_count + 1;
      end if;
   end procedure comprueba_salidas;

begin

   -- Instanciar y conectar UUT
   uut : component flipflopRS port map
         (q, notq, clk, R, S, reset_n);

   reset_n <= '1',
              '0' after (PERIODO/4),
              '1' after (PERIODO + PERIODO/4);

   clk <= not clk after (PERIODO/2);

   gen_vec_test : process is
      variable error_count : integer := 0; -- Nm. errores
   begin
      report "Comienza la simulacion";

      -- Vectores de test y comprobacion del resultado
      S <= '0';  R <= '0';  wait for PERIODO;       -- 1
      comprueba_salidas('0', q, notq, error_count);
      S <= '0';  R <= '1';  wait for PERIODO;       -- 2
      comprueba_salidas('0', q, notq, error_count);
      S <= '1';  R <= '0';  wait for PERIODO;       -- 3
      comprueba_salidas('1', q, notq, error_count);
      S <= '0';  R <= '0';  wait for PERIODO;       -- 4
      comprueba_salidas('1', q, notq, error_count);
      S <= '1';  R <= '0';  wait for PERIODO;       -- 5
      comprueba_salidas('1', q, notq, error_count);
      S <= '0';  R <= '1';  wait for PERIODO;       -- 6
      comprueba_salidas('0', q, notq, error_count);

      -- Informe final
      if (error_count = 0) then
         report "Simulacin finalizada sin errores.";
      else
         report "ERROR: Hay "               & 
                integer'image(error_count)  &
                " errores.";
      end if;
      wait; 
   end process gen_vec_test;

end architecture bp_flipflopRS;