-- Banco de pruebas del flip-flop D
-- fichero: bp_FFD_Enable_Set_Clear.vhd
library IEEE;
use IEEE.std_logic_1164.all;

entity bp_FFD is 
end entity bp_FFD;

architecture bp_FFD of bp_FFD is
   constant PERIODO               : time       := 100 ns;     -- Reloj
   signal   Q                     : std_logic;          -- Salidas UUT
   signal   Clock                 : std_logic  := '0'; -- Entradas UUT
   signal   Enable, Set, Clear, D : std_logic;      
  
   component FFD is
      port ( Q                             : out std_logic;
             Clock, Enable, Set, Clear, D  : in std_logic );
   end component FFD;

   -- Procedimiento para comprobar las salidas del flip-flop
   procedure comprueba_salidas
      ( esperado_q   :       std_logic;
        actual_q     :       std_logic;
        error_count  : inout integer    ) is
   begin
      -- Comprueba q
      if ( esperado_q /= actual_q ) then
         report "ERROR: Estado esperado ("     &
                std_logic'image(esperado_q)    &
                "), estado actual ("           &
                std_logic'image(actual_q)      &
                "), instante: "                &
                time'image(now);
         error_count := error_count + 1;
      end if;     
   end procedure comprueba_salidas;

begin

   -- Instanciar y conectar UUT
   uut : component FFD port map 
         ( Q, Clock, Enable, Set, Clear, D );
             
   Clock <= not Clock after (PERIODO/2); 
                           
   gen_vec_test : process is
      variable error_count : integer := 0; -- Nm. errores
   begin
      report "Comienza la simulacin";
    
      -- Vectores de test y comprobacin del resultado
      Set <= '1'; Clear <= '0'; Enable <= '0'; D <= '1';        -- 1
      wait for PERIODO; comprueba_salidas('1', Q, error_count);  
      Set <= '0'; Clear <= '0'; Enable <= '1'; D <= '0';        -- 2
      wait for PERIODO; comprueba_salidas('0', Q, error_count); 
      Set <= '0'; Clear <= '0'; Enable <= '0'; D <= '1';        -- 3
      wait for PERIODO; comprueba_salidas('0', Q, error_count); 
      Set <= '0'; Clear <= '0'; Enable <= '1'; D <= '1';        -- 4
      wait for PERIODO; comprueba_salidas('1', Q, error_count); 
      Set <= '0'; Clear <= '1'; Enable <= '1'; D <= '1';        -- 5
      wait for PERIODO; comprueba_salidas('0', Q, error_count);          
    
      report "ERROR: Hay "              &
             integer'image(error_count) &
             " errores.";
      wait;
   end process gen_vec_test;

end architecture bp_FFD;