-- Banco de pruebas de un contador ascendente sncrono mdulo 4
-- fichero: bp_contador_Mod_4.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use work.STATE_CONSTANTS.all;

entity bp_contador_Mod_4 is 
end entity bp_contador_Mod_4;

architecture bp_contador_Mod_4 of bp_contador_Mod_4 is

   constant PERIODO  : time       := 100 ns;           -- Reloj
   signal   q        : std_logic_vector(STATE_BITS-1 downto 0);  
   signal   clk      : std_logic  := '0';                        
   signal   reset, c : std_logic;           
  
 
   component contador_Mod_4 is
      port ( state  : out std_logic_vector(STATE_BITS-1 downto 0);
             clock, reset, c  : in std_logic );
   end component contador_Mod_4;

   -- Procedimiento para comprobar las salidas 
   procedure comprueba_salidas
      ( esperado_q   :       std_logic_vector(STATE_BITS-1 downto 0);
        actual_q     :       std_logic_vector(STATE_BITS-1 downto 0);
        error_count  : inout integer) is
   begin
      -- Comprueba q
      if ( esperado_q /= actual_q ) then
         report "ERROR: Estado esperado ("        &
                std_logic'image(esperado_q(1))    &
                std_logic'image(esperado_q(0))    & 
                "), estado actual ("              &
                std_logic'image(actual_q(1))      &
                std_logic'image(actual_q(0))      &
                "), instante: "                   &
                time'image(now);
         error_count := error_count + 1;
      end if;
   end procedure comprueba_salidas;

begin
   -- Instanciar y conectar UUT
   uut : component contador_Mod_4 port map
         (q,  clk, reset, c);
                  
   reset <=   '0', 
              '1' after (PERIODO/4),
              '0' after (PERIODO + PERIODO/4);

   clk <= not clk after (PERIODO/2);  

   gen_vec_test : process is
      variable error_count : integer := 0; -- Nm. errores
   begin
      report "Comienza la simulacin";

      -- Vectores de test y comprobacin del resultado
      C <= '0';   wait for PERIODO;          -- 1
      comprueba_salidas(S0, q, error_count); 
      C <= '1';   wait for PERIODO;          -- 2
      comprueba_salidas(S1, q, error_count);  
      C <= '0';   wait for PERIODO;          -- 3
      comprueba_salidas(S1, q, error_count); 
      C <= '1';   wait for PERIODO;          -- 4
      comprueba_salidas(S2, q, error_count); 
      C <= '0';   wait for PERIODO;          -- 5
      comprueba_salidas(S2, q, error_count); 
      C <= '1';   wait for PERIODO;          -- 6
      comprueba_salidas(S3, q, error_count); 
      C <= '0';   wait for PERIODO;          -- 7
      comprueba_salidas(S3, q, error_count); 
      C <= '1';   wait for PERIODO;          -- 8
      comprueba_salidas(S0, q, error_count);     

      -- Informe final
      if (error_count = 0) then
         report "Simulacin finalizada sin errores";
      else
         report  "ERROR: Hay "                & 
                 integer'image(error_count )  &
                 " errores.";
      end if;
      wait;
   end process gen_vec_test;

end architecture bp_contador_Mod_4;