-- Banco de pruebas del flip-flop JK
-- fichero: bp_flipflopJK.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity bp_flipflopJK is 
end entity bp_flipflopJK;

architecture bp_flipflopJK of bp_flipflopJK is
   constant PERIODO        : time       := 100 ns; -- Reloj
   signal   q, q_n         : std_logic;            -- Salidas UUT
   signal   clk            : std_logic  := '0';    -- Entradas UUT
   signal   J, K, reset_n  : std_logic;           
  
   component flipflop_JK is
      port ( q, q_n              : out std_logic;
             clk, J, K, reset_n  : in  std_logic  );
   end component flipflop_JK;

   -- Procedimiento para comprobar las salidas del flip-flop
   procedure comprueba_salidas
      ( esperado_q             :       std_logic;
        actual_q, actual_q_n   :       std_logic;
        error_count            : inout integer) is
   begin
      -- Comprueba q
      if ( esperado_q /= actual_q ) then
         report "ERROR: Estado esperado ("     &
                std_logic'image(esperado_q)    &
                "), estado actual ("           &
                std_logic'image(actual_q)      &
                "), instante: "                &
                time'image(now);
         error_count := error_count + 1;
      end if;
      -- Comprueba q_n
      if ( (not esperado_q) /= actual_q_n ) then
         report "ERROR: q_n esperado ("            &
                std_logic'image((not esperado_q))  &
                "), valor actual ("                &
                std_logic'image(actual_q_n)        &
                "), instante: "                    &
                time'image(now);
         error_count := error_count + 1;
      end if;       
   end procedure comprueba_salidas;

begin
   uut : component flipflop_JK port map  -- Instanciar y conectar UUT
         (q, q_n, clk, J, K, reset_n);
   reset_n <= '1',
              '0' after (PERIODO/4),
              '1' after (PERIODO+PERIODO/4);
   clk     <= not clk after (PERIODO/2);             
                  
   gen_vec_test : process is
      variable error_count : integer := 0; -- Nm. errores
   begin
      report "Comienza la simulacin";
      -- Vectores de test y comprobacin del resultado
      J <= '0'; K <= '0';   wait for PERIODO;   -- 1
      comprueba_salidas('0',q,q_n,error_count); 
      J <= '0'; K <= '1';   wait for PERIODO;   -- 2
      comprueba_salidas('0',q,q_n,error_count);  
      J <= '1'; K <= '0';   wait for PERIODO;   -- 3
      comprueba_salidas('1',q,q_n,error_count); 
      J <= '0'; K <= '1';   wait for PERIODO;   -- 4
      comprueba_salidas('0',q,q_n,error_count); 
      J <= '1'; K <= '1';   wait for PERIODO;   -- 5
      comprueba_salidas('1',q,q_n,error_count); 
      J <= '0'; K <= '0';   wait for PERIODO;   -- 6
      comprueba_salidas('1',q,q_n,error_count); 
      J <= '1'; K <= '0';   wait for PERIODO;   -- 7
      comprueba_salidas('1',q,q_n,error_count); 
      J <= '1'; K <= '1';   wait for PERIODO;   -- 8
      comprueba_salidas('0',q,q_n,error_count); 
      J <= '0'; K <= '0';  -- Deja el flip-flop en el estado '0'
      -- Informe final
      if (error_count = 0) then
         report "Simulacin finalizada sin errores";
      else
         report  "ERROR: Hay "              & 
                 integer'image(error_count) & " errores.";
      end if;
      wait;     -- Final del bloque process, pero no de la
                -- simulacin ya que la architecture tiene 
                -- sentencias de asignacin concurrente
   end process gen_vec_test;
end architecture bp_flipflopJK;