-- Mquina de Moore
-- fichero: maquinaMooreSimple.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use work.STATE_CONSTANTS.all;

entity maquinaMooreSimple is
   port( z       : out std_logic;  -- Seal de salida
         state   : out std_logic_vector (STATE_BITS-1 downto 0);
                                   -- Estado actual de la mquina
         reset_n : in  std_logic;  -- Seal reset activada en bajo
         clk     : in  std_logic;  -- Seal de reloj
         x       : in  std_logic); -- Seal de entrada
end entity maquinaMooreSimple;

architecture maquinaMooreSimple of maquinaMooreSimple is
   signal internal_state: std_logic_vector(STATE_BITS-1 downto 0);
begin
   state <= internal_state; -- Muestra el estado
   
   -- Genera la salida
   salida: process(internal_state) is
   begin
      case internal_state is
         when S0     =>   z <= '0';
         when S1     =>   z <= '0';
         when S2     =>   z <= '1';
         when others =>   z <= 'X';
      end case;
   end process salida;
   
   -- Genera el siguiente estado
   proximo_estado: process(reset_n,clk) is
   begin
      if (reset_n = '0') then      -- Reset asncrono
         internal_state <= S0;
      elsif rising_edge(clk) then  -- En flanco subida del reloj
         case internal_state is
            when S0 =>             -- Estado actual: S0
               if (x = '1') then
                  internal_state <= S1;
               else
                  internal_state <= S2;
               end if;
            when S1 =>             -- Estado actual: S1
               if (x = '0') then
                  internal_state <= S2;
               end if;
            when S2 =>             -- Estado actual: S2 
               if (x = '0') then
                  internal_state <= S0;
               end if;
            when others =>         -- Por completitud
               internal_state <= "XX"; 
         end case;
      end if;   
   end process proximo_estado;
end architecture maquinaMooreSimple;