-- Mquina de Mealy detectora de la secuencia 0110
-- fichero: seq0110_ffD.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use work.seq0110_CONST.all;

entity seq0110 is
   port ( z_long  : out std_logic;   -- Seal de salida
          state   : out std_logic_vector(STATE_BITS-1 downto 0); 
                                     -- Estado actual
          reset_n : in  std_logic;   -- Reset sncrono activado LOW   
          clk     : in  std_logic;   -- Seal de reloj
          x       : in  std_logic ); -- Bit de entrada                                     
end entity seq0110;

architecture seq0110 of seq0110 is
   signal internal_state : std_logic_vector(STATE_BITS-1 downto 0);
   signal z  : std_logic;  -- Seal de salida obtenida a partir 
                           -- del estado actual y la entrada actual,
                           -- mediante un circuito combinacional
begin
   state <= internal_state;
    
   -- Clculo de la salida, que se hace pasar por un flip-flop D para  
   -- garantizar que se mantiene su valor durante un ciclo de reloj
   z <= '1' when ( (internal_state = S3) and (x = '0') ) else '0';
        
   ffD_salida: process (clk) is
   begin
      if ( rising_edge(clk) ) then
         z_long <= z;
      end if;
   end process ffD_salida;
    
   -- Clculo del prximo estado
   proximo_estado: process (clk) is
   begin
      if ( (reset_n = '0') and rising_edge(clk) ) then -- Reset 
                                                       -- sncrono
         internal_state <= S0;
      elsif rising_edge(clk) then   -- Flanco de subida del reloj
         case internal_state is
            when S0 =>
               if (x = '0') then    -- Primer bit del patrn
                  internal_state <= S1;
               else
                  internal_state <= S0;
               end if;
            when S1 =>
               if (x = '1') then    -- Segundo bit del patrn
                  internal_state <= S2;
               else
                  internal_state <= S1;
               end if;
            when S2 =>
               if (x = '1') then    -- Tercer bit del patrn
                  internal_state <= S3;
               else
                  internal_state <= S1;
               end if;
            when others =>
               if (x = '0') then    -- Cuarto bit del patrn
                  internal_state <= S1;
               else
                  internal_state <= S0;
               end if;               
         end case;   
      end if;
   end process proximo_estado; 

end architecture seq0110;