-- Banco de pruebas del conversor de BCD a binario
-- fichero: bp_bcd_to_bin.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.BCD_CONST.all;

entity bp_bcd_to_bin is
   constant MAX_BCD : integer := 9999; -- Valor mximo de entrada
   constant DELAY   : time    := 10ns; -- Retardo usado en el test;
end entity bp_bcd_to_bin;

architecture bp_bcd_to_bin of bp_bcd_to_bin is
   signal bin: std_logic_vector(LENGTH-1 downto 0); -- Salida de UUT
   signal bcd: std_logic_vector(LENGTH-1 downto 0); -- Entrada a UUT
   
   -- Declaracin del componente (UUT)
   component bcd_to_bin is 
       port( bin: out std_logic_vector(LENGTH-1 downto 0);
             bcd: in  std_logic_vector(LENGTH-1 downto 0) );    
   end component bcd_to_bin;
   
begin
   -- Instancia la unidad a testear
   UUT : component bcd_to_bin port map(bin,bcd);
       
   -- Genera vectores de test y comprueba resultados
   main: process is
      variable temp   : unsigned (LENGTH-1 downto 0);
      variable digit  : unsigned (LENGTH-1 downto 0); -- Un dgito BCD
      variable digits : unsigned (LENGTH-1 downto 0); -- Todos dgitos
      variable expected : unsigned (LENGTH-1 downto 0);
      variable error_count : integer  := 0;           -- Nm. errores
   begin
      report "Comienza la simulacin.";  
     
      -- Generar todos los posibles valores
      for i in 0 to MAX_BCD loop
         temp := TO_UNSIGNED(i,LENGTH);
         for j in 0 to (LENGTH/4-1) loop
            digit := temp mod 10;   -- Dgito BCD menos significativo
            digits(j*4+3 downto j*4) := digit(3 downto 0);
            temp := temp/10;
         end loop;
         bcd <= std_logic_vector(digits);     -- Asigna vector de test
         expected := TO_UNSIGNED(i, LENGTH);          
         wait for DELAY;
         assert (expected = unsigned(bin))
            report "ERROR: Resultado esperado "             & 
                   integer'image(TO_INTEGER(expected))      &
                   ", resultado obtenido "                  &
                   integer'image(TO_INTEGER(unsigned(bin))) &
                   " en el instante "                       & 
                   time'image(now);
         if (expected /= unsigned(bin)) then
            error_count := error_count + 1;
         end if;
      end loop;
       
      wait for DELAY;
      -- Informe final
      if (error_count = 0) then
         report "Simulacin finalizada sin errores";
      else 
         report "Hay " &  integer'image(error_count) & " errores.";
      end if;
      wait;     -- Final de la simulacin
   end process main;

end architecture bp_bcd_to_bin;