-- Banco de pruebas del circuito que calcula la exponenciacin
-- de dos nmeros binarios: z = x ** y
-- fichero: bp_exponen.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use work.exponen_CONST.all;

entity bp_exponen is
   constant PERIODO : time := 100 ns; -- Periodo del reloj
end entity bp_exponen;

architecture bp_exponen of bp_exponen is

   -- Declaracin de seales para conectar la UUT
   signal z       : std_logic_vector (MAX_DATA_BITS-1 downto 0);
   signal paso    : std_logic_vector (BITS_PASO-1 downto 0);
   signal reset_n : std_logic; 
   signal clk     : std_logic;
   signal start   : std_logic;
   signal x       : std_logic_vector(MAX_DATA_BITS-1 downto 0);
   signal y       : std_logic_vector(MAX_DATA_BITS-1 downto 0);
   
   -- Declaracin de otras seales
   signal fin     : boolean := false;
   
   component exponen is 
      port ( z       : out std_logic_vector (MAX_DATA_BITS-1 downto 0);
             paso    : out std_logic_vector (BITS_PASO-1 downto 0);
             reset_n : in  std_logic;  -- Reset asncrono activado LOW
             clk     : in  std_logic;
             start   : in  std_logic;
             x       : in  std_logic_vector(MAX_DATA_BITS-1 downto 0);
             y       : in  std_logic_vector(MAX_DATA_BITS-1 downto 0));
   end component exponen;
     
begin

   uut : component exponen port map
         (z, paso, reset_n , clk , start, x , y);

   -- Pulso de reset
   reset_n <= '1', 
              '0' after (PERIODO/4), 
              '1' after (PERIODO + PERIODO/4);

   -- Seal de reloj
   reloj : process is
   begin
      clk <= '0'; wait for (PERIODO/2);
      clk <= '1'; wait for (PERIODO/2);
      if (fin) then
          wait;   -- Termina este proceso
      end if;
   end process reloj;

   -- Generacin de los vectores de test
   genera_vectores : process is
   begin
      wait until falling_edge(clk);
      
      -- Test 1: x vale 0, y vale 1
      x <= (others => '0');  y <= (0 => '1', others => '0');
      wait for PERIODO;
      start <= '1';                  -- Comienza el clculo x**y
      wait until (z = B"0000_0000"); -- Espera hasta obtener z=0
      
      -- Test 2: x vale 1, y vale 0
      x <= (0 => '1', others => '0');  y <= (others => '0');
      wait for PERIODO;
      start <= '1';  -- Comienza el clculo x**y
      wait until (z = B"0000_0001"); -- Espera hasta obtener z=1
      
      -- Test 3: x vale 0000_0011, y vale 0000_0010
      x <= "00000011";  y <= "00000010";
      wait for PERIODO;
      start <= '1';  -- Comienza el clculo x**y
      wait until (z = B"0000_1001"); -- Espera hasta obtener z=9
      
      -- Final de la simulacin
      report "Fin de la simulacin";
      fin <= true;                   -- Finaliza el proceso del reloj
      wait;                          -- Finaliza este proceso
   end process genera_vectores;

end architecture bp_exponen;