-- Operacin exponenciacin: z = x ** y
-- fichero: exponen.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.exponen_CONST.all;

entity exponen is 
   port ( z       : out std_logic_vector(MAX_DATA_BITS-1 downto 0);
                        -- Resultado de la exponenciacin
          paso    : out std_logic_vector(BITS_PASO-1 downto 0);
                        -- Paso en la ejecucin del algoritmo
          reset_n : in  std_logic;  -- Reset asncrono activado LOW
          clk     : in  std_logic;
          start   : in  std_logic;
          x       : in  std_logic_vector(MAX_DATA_BITS-1 downto 0);
          y       : in  std_logic_vector(MAX_DATA_BITS-1 downto 0) );
end entity exponen;

architecture exponen of exponen is
begin

   seudocodigo: process (reset_n, clk) is
      variable resultado : unsigned(MAX_DATA_BITS*2-1 downto 0);
      variable base      : unsigned(MAX_DATA_BITS-1 downto 0);
      variable expo      : unsigned(MAX_DATA_BITS-1 downto 0);
      variable paso_v    : std_logic_vector (BITS_PASO-1 downto 0);
                            -- Paso en la ejecucin del algoritmo
   begin
      if (reset_n = '0') then
         paso_v := PASO1;             -- Al resetear, ir al Paso 1
      elsif rising_edge(clk) then
         case paso_v is
            when PASO1 =>             -- Inicializacin
               base := unsigned(x);
               expo := unsigned(y);
               if (start = '1') then
                  if (x = B"0000_0000") then 
                     resultado := ( others => '0' );
                     paso_v := PASO3;
                  elsif (y = B"0000_0000") then
                     resultado := ( 0 => '1', others => '0' );
                     paso_v := PASO3;
                  else
                     resultado := ( 0 => '1', others => '0' );
                     paso_v := PASO2;
                  end if;
               end if;
            when PASO2 =>             -- Repetir las multiplicaciones
               resultado := resultado(MAX_DATA_BITS-1 downto 0)*base;
                                   -- 8 bits * 8 bits = 16 bits
               expo := expo - 1;
               if (expo = 0) then
                  paso_v := PASO3; -- Termina
               else
                  paso_v := PASO2; -- Multiplica de nuevo
               end if;
            when PASO3 =>             -- Almacena el resultado   
               z <= std_logic_vector(
                       resultado(MAX_DATA_BITS-1 downto 0)  );
               paso_v := PASO1;
            when others =>
               null;             
         end case;
         paso <= paso_v;  
      end if;                              
   end process seudocodigo;

end architecture exponen;