-- Banco de pruebas para el circuito de la mquina expendedora
-- fichero: bp_maquinaExpend.vhd
library IEEE;
use IEEE.std_logic_1164.all;   
use IEEE.numeric_std.all;
use work.maqExpend_defs.all;

entity bp_maquinaExpend is
end entity bp_maquinaExpend;

architecture bp_maquinaExpend of bp_maquinaExpend is

   component maquinaExpend is
      port ( total             : out unsigned(TOTAL_BITS-1 downto 0);  
             ack               : out std_logic;                        
             beep              : out std_logic;          
             devuelve_moneda   : out std_logic;
             dispensa_refresco : out std_logic;
             dispensa_agua     : out std_logic;
             devuelve_cambio   : out std_logic;
             req               : in  std_logic;
             data              : in  unsigned(DATA_BITS-1 downto 0);
             reset_n           : in  std_logic;
             clk               : in  std_logic );
   end component maquinaExpend;

   constant PERIODO : time := 100 ns; -- Periodo seal de reloj
  
   signal  total             : unsigned(TOTAL_BITS-1 downto 0);  
   signal  ack               : std_logic;                        
   signal  beep              : std_logic;          
   signal  devuelve_moneda   : std_logic;
   signal  dispensa_refresco : std_logic;
   signal  dispensa_agua     : std_logic;
   signal  devuelve_cambio   : std_logic;
   signal  req               : std_logic;
   signal  data              : unsigned(DATA_BITS-1 downto 0);
   signal  reset_n           : std_logic;
   signal  clk               : std_logic;
  
begin
    
   uut : maquinaExpend port map
         ( total, ack ,beep, devuelve_moneda, 
           dispensa_refresco, dispensa_agua, 
           devuelve_cambio, req, data, reset_n, clk );      
  
   -- Crea un pulso de reset
   reset_n <= '1',
              '0' after (PERIODO*3/4),
              '1' after (2*PERIODO);
             
   -- Seal de reloj
   clock : process is 
   begin
      clk <= '0'; wait for (PERIODO/2);
      clk <= '1'; wait for (PERIODO/2);
   end process clock;               
  
   -- Vectores de test
   generar_vectores: process is
   begin
      
      req <= '0';
      data <= (others => '0');
      wait for (PERIODO*3);
      wait for (PERIODO/4);
      
      req <= '1'; data <= DATA_m50;    -- Introduce moneda de 50 cent.
      wait for (PERIODO*5);
      req <= '0'; wait for (PERIODO*2);
      
      req <= '1'; data <= DATA_m20;    -- Introduce moneda de 20 cent.
      wait for (PERIODO*3);
      req <= '0'; wait for (PERIODO*2);
      
      req <= '1'; data <= DATA_selec_agua;          -- Selecciona agua
      wait for (PERIODO*3);
      req <= '0'; wait for (PERIODO*2);
      
      req <= '1'; data <= DATA_m50;    -- Introduce moneda de 50 cent.
      wait for (PERIODO*3);
      req <= '0'; wait for (PERIODO*2);
      
      req <= '1'; data <= DATA_m10;    -- Introduce moneda de 10 cent.
      wait for (PERIODO*3);
      req <= '0'; wait for (PERIODO*2);

      req <= '1'; data <= DATA_selec_refresco;  -- Selecciona refresco
      wait for (PERIODO*3);
      req <= '0'; wait for (PERIODO*2);

      req <= '1'; data <= DATA_m20;    -- Introduce moneda de 20 cent.
      wait for (PERIODO*3);
      req <= '0'; wait for (PERIODO*2);

      req <= '1'; data <= DATA_selec_refresco;  -- Selecciona refresco
      wait for (PERIODO*3);
      req <= '0'; wait for (PERIODO*2);

      req <= '1'; data <= DATA_devolucion;    -- Devolucin del dinero
      wait for (PERIODO*3);
      req <= '0'; wait for (PERIODO*2);

      report "Final de la simulacin";
      wait;
      
   end process generar_vectores;
        
end architecture bp_maquinaExpend;