-- Circuito de la mquina expendedora
-- fichero: maquinaExpend.vhd
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.maqExpend_defs.all;

entity maquinaExpend is
   port ( total             : out unsigned(TOTAL_BITS-1 downto 0);  
          ack               : out std_logic;                        
          beep              : out std_logic;          
          devuelve_moneda   : out std_logic;
          dispensa_refresco : out std_logic;
          dispensa_agua     : out std_logic;
          devuelve_cambio   : out std_logic;
          req               : in  std_logic;
          data              : in  unsigned(DATA_BITS-1 downto 0);
          reset_n           : in  std_logic;  -- Reset asncrono 
                                              -- activado LOW
          clk               : in  std_logic );
end entity maquinaExpend;

architecture maquinaExpend of maquinaExpend is
   signal total_int : unsigned(TOTAL_BITS-1  downto 0);
   signal estado    : unsigned(ESTADO_BITS-1 downto 0);
begin
   total <= total_int;
   
   algoritmo : process (reset_n, clk) is
   begin
      if (reset_n = '0') then
         total_int <= (others => '0'); -- Inicializa el registro
         ack               <= '0';                        
         beep              <= '0';         
         devuelve_moneda   <= '0';
         dispensa_refresco <= '0';
         dispensa_agua     <= '0';
         devuelve_cambio   <= '0';
         estado            <= "00";    -- Estado inicial
      elsif rising_edge(clk) then
         estado <= estado + 1;         -- Actualiza estado para el 
                                       -- siguiente pulso de reloj
         case estado is                -- Los pasos del algoritmo se 
                                       -- codifican como estados
            when "00" =>
               if (req = '0') then     -- Espera hasta que (req='1')
                  estado <= "00";
               end if;
            when "01" =>
               case (data) is
                  when DATA_m10 | DATA_m20 | DATA_m50 =>
                     if ( (total_int + data) > MAX ) then
                        devuelve_moneda <= '1';
                     else 
                        total_int <= total_int + data;
                     end if;
                  when DATA_selec_refresco =>
                     if (total_int < 6) then
                        beep <= '1';
                     else
                        total_int <= total_int - 6;
                        dispensa_refresco <= '1';
                     end if;
                  when DATA_selec_agua =>
                     if (total_int < 7) then
                        beep <= '1';
                     else
                        total_int <= total_int - 7;
                        dispensa_agua <= '1';
                     end if;
                  when DATA_devolucion =>
                     total_int <= (others => '0');
                     devuelve_cambio <= '1';
                  when others =>
                     assert false report "Caso indefinido.";
               end case;
            when "10" =>
               ack <= '1';
               if (req = '1') then     -- Espera hasta que (req='0')
                  estado <= "10";
               end if;
            when others =>
               ack               <= '0';                        
               beep              <= '0';         
               devuelve_moneda   <= '0';
               dispensa_refresco <= '0';
               dispensa_agua     <= '0';
               devuelve_cambio   <= '0'; 
         end case; -- case estado
      end if;
   end process algoritmo;
    
end architecture maquinaExpend; 